package edu.nyu.scps.capture;

/*
 * The application needs to have the permission to write to external storage
 * if the output file is written to the external storage, and also the
 * permission to record audio. These permissions must be set in the
 * application's AndroidManifest.xml file, with something like:
 *
 * <uses-permission android:name="android.permission.WRITE_EXTERNAL_STORAGE" />
 * <uses-permission android:name="android.permission.RECORD_AUDIO" />
 *
 */

import java.io.File;

import android.app.Activity;
import android.media.MediaPlayer;
import android.media.MediaPlayer.OnCompletionListener;
import android.media.MediaRecorder;
import android.os.Bundle;
import android.os.Environment;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Toast;
import android.widget.ToggleButton;

public class CaptureActivity extends Activity {

	private static String fileName =
		Environment.getExternalStorageDirectory().getAbsolutePath() + "/capture.3gp";

	private ToggleButton recordButton;
	private MediaRecorder mediaRecorder;

	private ToggleButton playButton;
	private MediaPlayer mediaPlayer;

	@Override
	public void onCreate(Bundle icicle) {
		super.onCreate(icicle);
		setContentView(R.layout.main);
		
		recordButton = (ToggleButton)findViewById(R.id.recordButton);
		playButton = (ToggleButton)findViewById(R.id.playButton);
		recordButton.clearFocus();

		recordButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (((ToggleButton)v).isChecked()) {
					
					if (mediaPlayer != null && mediaPlayer.isPlaying()) {
						mediaPlayer.stop();
						playButton.setChecked(false);
					}

					if (mediaRecorder == null) {
						mediaRecorder = new MediaRecorder();

						mediaRecorder.setOnErrorListener(new MediaRecorder.OnErrorListener() {
							public void onError(MediaRecorder mr, int what, int extra) {
								String s =
									what == MediaRecorder.MEDIA_RECORDER_ERROR_UNKNOWN ? "MEDIA_RECORDER_ERROR_UNKNOWN" :
									Integer.toString(what);
						
								Toast.makeText(CaptureActivity.this, s + " " + extra, Toast.LENGTH_LONG).show();
							}
						});
					}
					
					try {
						mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);	//microphone
						mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
						mediaRecorder.setOutputFile(fileName);
						//Adaptive Multi-Rate audio codec, narrow band
						mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);
						mediaRecorder.prepare();
					} catch (Exception e) {
						Toast.makeText(CaptureActivity.this, e.toString(), Toast.LENGTH_LONG);
					}

					mediaRecorder.start();
					playButton.setEnabled(true);
				} else {
					mediaRecorder.stop();
				}
			}
		});
		
		playButton.setOnClickListener(new OnClickListener() {
			public void onClick(View v) {
				if (((ToggleButton)v).isChecked()) {
					if (mediaRecorder != null) {
						try {
							mediaRecorder.stop();
						}
						catch (IllegalStateException e) {
							Toast.makeText(CaptureActivity.this, e.toString(), Toast.LENGTH_LONG);
						}
						recordButton.setChecked(false);
					}

					if (mediaPlayer != null) {
						mediaPlayer.release();
					}
					mediaPlayer = new MediaPlayer();

					try {
						mediaPlayer.setDataSource(fileName);
					} catch (Exception e) {
						Toast.makeText(CaptureActivity.this, e.toString(), Toast.LENGTH_LONG);
					}

					mediaPlayer.setOnErrorListener(new MediaPlayer.OnErrorListener() {
						public boolean onError(MediaPlayer mp, int what, int extra) {
							String s =
								what == MediaPlayer.MEDIA_ERROR_UNKNOWN ? "MEDIA_ERROR_UNKNOWN" :
								what == MediaPlayer.MEDIA_ERROR_SERVER_DIED ? "MEDIA_ERROR_SERVER_DIED" :
								Integer.toString(what);
					
							Toast.makeText(CaptureActivity.this, s + " " + extra, Toast.LENGTH_LONG).show();
							return true;
						}
					});

					mediaPlayer.setOnCompletionListener(new OnCompletionListener() {
						public void onCompletion(MediaPlayer mp) {
							mediaPlayer.stop();
							playButton.setChecked(false);
							recordButton.setEnabled(true);
						}
					});
					
					try {
						mediaPlayer.prepare();
					} catch (Exception e) {
						Toast.makeText(CaptureActivity.this, e.toString(), Toast.LENGTH_LONG);
					}

					mediaPlayer.start();
				} else {
					mediaPlayer.stop();
				}
			}
		});

		Toast.makeText(this, fileName, Toast.LENGTH_LONG).show();
	}

	@Override
	public void onDestroy() {
		if (mediaRecorder != null) {
			mediaRecorder.release();
			mediaRecorder = null;
		}

		if (mediaPlayer != null) {
			mediaPlayer.release();
			mediaPlayer = null;
		}

		final File file = new File(fileName);
		if (file.exists()) {
			file.delete();
		}
		super.onDestroy();
	}
}